<?php

App::uses('AppBehavior', 'Model/Behavior');

class CalculateTaxBehavior extends AppBehavior {

	/**
	 * Store the tax Rates if handled by another table
	 */
	protected $_taxRates = array();

	/**
	 * Initialise the behavior - store the settings against the model
	 *
	 * @see ModelBehavior::setup()
	 */
	public function setup(Model $Model, $settings = array()) {
		$defaults = array(
			'fields' => array(),
			'TaxRateModel' => false,
			'TaxRateModelId' => false
		);

		$this->settings[$Model->alias] = array_merge(
			$defaults,
			$settings
		);
	}

	/**
	 * After find callback. Can be used to modify any results returned by find.
	 *
	 * @param Model $model Model using this behavior
	 * @param mixed $results The results of the find operation
	 * @param bool $primary Whether this model is being queried directly (vs. being queried as an association)
	 * @return mixed An array value will replace the value of $results - any other value will be ignored.
	 */
	public function afterFind(Model $Model, $results, $primary = false) {
		if (empty($this->settings[$Model->alias]['fields']) || ! is_array($this->settings[$Model->alias]['fields'])) {
			return $results;
		}

		if (! empty($this->settings[$Model->alias]['TaxRateModel'])) {
			$TaxRateModel = EvClassRegistry::init($this->settings[$Model->alias]['TaxRateModel']);
		}

		foreach ($results as $key => $item) {

			if (! empty($item['TaxLevel']['rate'])) {
				$rate = $item['TaxLevel']['rate'];
			} elseif (isset($TaxRateModel)) {

				$rate = $this->_getRate($Model, $TaxRateModel, $item);

				foreach ($this->settings[$Model->alias]['fields'] as $field) {
					if (! array_key_exists($field, $item[$Model->alias])) {
						continue;
					}

					$results[$key][$Model->alias][$field . '_incTax'] = $item[$Model->alias][$field] + ($item[$Model->alias][$field] * ($rate / 100));
				}
			}
		}

		return $results;
	}

	/**
	 * work out the tax rate
	 * - either query table
	 * - retrieve from stored array
	 *
	 * @param 	object 			Model object of the original model that called this behavior
	 * @param 	object 			Model Object of the related model that has the tax rate id
	 * @param 	array 			data array
	 * @return 	decimal 		decimal rate, will return zero if none found
	 */
	protected function _getRate($Model, $TaxRateModel, $item) {
		$modelId = Hash::get($item, $this->settings[$Model->alias]['TaxRateModelId']);

		if (! empty($this->_taxRates[$modelId])) {
			return $this->_taxRates[$modelId];
		}

		if (! empty($modelId)) {
			$rate = $TaxRateModel->getTaxRate($modelId);

			if (! empty($rate)) {
				$this->_taxRates[$modelId] = $rate;
				return $rate;
			}
		}

		return intval(0);
	}
}
