<?php

App::uses('AppBehavior', 'Model/Behavior');

class TaxableBehavior extends AppBehavior {

	/**
	 * beforeFind can be used to cancel find operations, or modify the query that will be executed.
	 * By returning null/false you can abort a find. By returning an array you can modify/replace the query
	 * that is going to be run.
	 *
	 * @param Model $model Model using this behavior
	 * @param array $query Data used to execute this query, i.e. conditions, order, etc.
	 * @return bool|array False or null will abort the operation. You can return an array to replace the
	 *   $query that will be eventually run.
	 */
	public function beforeFind(Model $Model, $query) {
		$Model->bindModel(
			array(
				'belongsTo' => array(
					'TaxLevel' => array(
						'className' => 'EvTax.TaxLevel'
					)
				)
			)
		);

		return $query;
	}

	/**
	 * get tax rate
	 *
	 * @param 	Model 			Model object auto populated by CakeORM
	 * @param 	int 			item ID
	 * @return 	decimal|bool 	Tax Rate or bool false if not found
	 */
	public function getTaxRate($Model, $itemId) {
		$data = $Model->find(
			'first',
			array(
				'conditions' => array(
					$Model->alias . '.id' => $itemId
				),
				'contain' => array(
					'TaxLevel'
				)
			)
		);

		if (isset($data['TaxLevel']) && array_key_exists('rate', $data['TaxLevel'])) {
			return $data['TaxLevel']['rate'];
		}

		return false;
	}
}
